//*****************************************************************************
// File Name            : iobus2wb.v
//-----------------------------------------------------------------------------
// Function             : MicroBlaze_MAC Iobus to Wishbone BUS 
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2011/12/17
//*****************************************************************************
module iobus2wb (
                 
Clk, Reset, IO_Ready, IO_Addr_Strobe, IO_Read_Strobe, IO_Write_Strobe,  IO_Read_Data, IO_Address, IO_Byte_Enable, IO_Write_Data,
CLK_O, RST_O, CYC_O, STB_O, ADR_O, SEL_O, WE_O, DAT_O, DAT_I, ACK_I ,ERR_I,RTY_I //wb 
);
// 
parameter p_wb_addr_low  =  32'hD0000000;
parameter p_wb_addr_hi   =  32'hEFFFFFFF;
//   
input Clk;
input Reset;
//IO bus
input IO_Addr_Strobe;
input IO_Read_Strobe;
input IO_Write_Strobe;
input [31 : 0] IO_Address;
input [3 : 0] IO_Byte_Enable;
input [31 : 0] IO_Write_Data;
output [31 : 0] IO_Read_Data;
output IO_Ready;
//WishBone Bus
output         CLK_O;         // clock
output         RST_O;         // reset (asynchronous active low)
output         CYC_O;         // cycle
output         STB_O;         // strobe
output  [31:0] ADR_O;         // address
output  [3:0]  SEL_O;         // byte_sel
output         WE_O;          // write enable
output  [31:0] DAT_O;         // data output
input   [31:0] DAT_I;         // data input
input          ACK_I;         // normal bus termination
input          ERR_I;         // error bus termination
input          RTY_I;         // retry
//
//
reg [31:0] IO_Read_Data;
reg IO_Ready;
   
reg         CYC_O;         // cycle
wire        STB_O;         // strobe
reg  [31:0] ADR_O;         // address
reg  [3:0]  SEL_O;         // byte_sel
reg         WE_O;          // write enable
reg  [31:0] DAT_O;         // data output
//
wire addr_hit;
wire wb_term;
   
assign    CLK_O = Clk; 
assign    RST_O = Reset;
//decode address
assign addr_hit  =  ((IO_Address>=p_wb_addr_low)&&(IO_Address<=p_wb_addr_hi))? 1'b1:1'b0;
//WishBoen Bus termination
assign wb_term = ((ACK_I==1'b1)||(ERR_I==1'b1)||(RTY_I==1'b1))?1'b1:1'b0;

// 
always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    CYC_O <= 1'b0;
  else 
    if (wb_term==1'b1)
       CYC_O <=  1'b0 ;
    else if (((IO_Read_Strobe==1'b1)||(IO_Write_Strobe==1'b1))&&(addr_hit==1'b1))
       CYC_O <=  1'b1 ;
    else
       CYC_O <=  CYC_O ;

assign STB_O = CYC_O;
      
always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    WE_O <= 1'b0;
  else 
    if (wb_term==1'b1)
       WE_O <=  1'b0 ;
    else if ((IO_Write_Strobe==1'b1)&&(addr_hit==1'b1))
       WE_O <=  1'b1 ;
    else
       WE_O <=  WE_O ;

always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    ADR_O <= 32'h00000000;
  else 
    if (IO_Addr_Strobe==1'b1)
       ADR_O <= IO_Address[31:0]  ;
    else
       ADR_O <=  ADR_O ;
   
always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    SEL_O <= 4'b0000;
  else 
    if (IO_Addr_Strobe==1'b1)
       SEL_O <=  IO_Byte_Enable  ;
    else
       SEL_O <= SEL_O ;

always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    IO_Ready <= 1'b0;
  else
    if (wb_term==1'b1)      
        IO_Ready <=  1'b1 ;
    else
        IO_Ready <= 1'b0;

always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    DAT_O <= 32'h00000000;
  else 
    if ((IO_Write_Strobe==1'b1)&&(addr_hit==1'b1))
       DAT_O <= IO_Write_Data;  
    else
       DAT_O <= DAT_O ;
        
always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    IO_Read_Data <= 32'h00000000;
  else 
    if (wb_term==1'b1)
      IO_Read_Data <= DAT_I;
    else
      IO_Read_Data <= IO_Read_Data ;
        
endmodule